#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：automation_nep
@File ：setup.py.py
@Author ：RongYi
@Date ：2025/4/29 14:32
@E-mail ：2071914258@qq.com
"""
# setup.py
from setuptools import setup, find_packages

setup(
    name="automation_nep",
    version="0.1",
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'auto_nep=cli.cli:main',  # 更新入口点为 main 函数
        ],
    },
)


#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：automation_nep
@File ：train.py
@Author ：RongYi
@Date ：2025/4/29 14:19
@E-mail ：2071914258@qq.com
"""

from utils.myprint import myprint
from dataset.abacus import abacus_dataset
from dataset.nep import abacus2xyz
from utils.task import sub_task, check_task
from train.nep import train_nep_v0


def train(config):
    """
    训练逻辑实现 分为 abacus gpumd nep 三个板块
    :param env_dict:
    :param config:
    :return:
    """
    if config['task_type'] == 'abacus':
        myprint("abacus 单点能计算开始")
        dataset_roots = abacus_dataset(config)  # 生成数据集 返回数据集大小
        sub_task(config, dataset_roots)  # 提交任务
        check_task(config, dataset_roots)  # 任务监测
        abacus2xyz(config["abacus"]["dataset_path"])  # 单点能提取
        train_nep_v0(config)  # nep-v0 训练



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：automation_nep
@File ：nep.py
@Author ：RongYi
@Date ：2025/4/30 11:25
@E-mail ：2071914258@qq.com
"""
import os
import re
from tqdm import tqdm
from utils.myprint import myprint


def abacus2xyz(abacus_dataset_path):
    """
    abacus 训练集 -> nep 训练集
    :param abacus_dataset_path:
    :return:
    """
    myprint("正在生成 nep 训练 trian.xyz 文件")
    # 重置 train.xyz
    with open("./nep-dataset/train.xyz", 'w', encoding='utf-8') as f:
        f.close()
    os.makedirs(abacus_dataset_path, exist_ok=True)
    log_files = [os.path.join(root, file) for root, _, files in os.walk(abacus_dataset_path) for file in files if
                 file == "running_scf.log"]

    for log_file in tqdm(log_files):
        with open(log_file, encoding='utf-8') as f:
            content = f.read()
        if "charge density convergence is achieved" not in content:
            myprint(f"任务未收敛: {log_file}", "RED")
            continue

        # config_type 训练集位置
        config_type = os.path.dirname(log_file).split('\\')[1:2]
        config_type = ''.join(config_type)

        # 提取原子总数
        total_atom_number_match = re.search(r"TOTAL ATOM NUMBER = (\d+)", content)
        total_atom_number = total_atom_number_match.group(1)

        # 提取晶格常数
        lattice_match = re.search(r" Lattice vectors.*(\n.*\n.*\n.*)", content)
        lattice = lattice_match.group(1).strip().replace('+', '')
        lattice = ' '.join([f"{float(l):.10f}" for l in lattice.split()])

        # 提取能量
        energy_match = re.search(r"FINAL_ETOT_IS\s+(\S+)", content)
        energy = float(energy_match.group(1))

        # 应力
        stress_match = re.search(r"TOTAL-STRESS.*\n.*(\n.*\n.*\n.*)", content)
        stress = stress_match.group(1).strip()
        stress = ' '.join([f"{float(s) * 0.1:.10f}" for s in stress.split()])

        # 位置和力 分数坐标 -> 笛卡尔坐标
        # 读取 a b c
        cell_a_match = re.search(r"NORM_A\s+\S+ (.*)", content)
        cell_b_match = re.search(r"NORM_B\s+\S+ (.*)", content)
        cell_c_match = re.search(r"NORM_C\s+\S+ (.*)", content)

        cell_a = float(cell_a_match.group(1))
        cell_b = float(cell_b_match.group(1))
        cell_c = float(cell_c_match.group(1))

        # 元素类型和坐标转换
        type_position_match = re.compile(r"taud_([a-z,A-z]+)\w+\s+([\d.]+)\s+([\d.]+)\s+([\d.]+)")
        type_position = type_position_match.findall(content)
        type_position = [f"{type}\t{float(x) * cell_a:.10f}\t{float(y) * cell_b:.10f}\t{float(z) * cell_c:.10f}" for type, x, y, z in type_position]
        # 力 找到TOTAL-FORCE开头后面,所有这类格式的行
        force_match = re.compile(
    r"TOTAL-FORCE \(eV/Angstrom\)\s*"
    r"------------------------------------------------------------------------------------------\s*"
    r"((?:\s*\S+\s+-?\d+\.\d+\s+-?\d+\.\d+\s+-?\d+\.\d+\s*\n)*)",
    re.DOTALL
)
        force = force_match.search(content).group(1)
        force = list(filter(None, force.split('\n')))
        # 输出文件
        with open("./nep-dataset/train.xyz", 'a', encoding='utf-8') as f:
            f.write(f"{total_atom_number}\n")  # 总原子数
            #表头
            f.write(f"Energy={energy:.8f} Lattice=\"{lattice}\" Stress=\"{stress}\""
                    f" Config_type={config_type} Properties=species:S:1:pos:R:3:forces:R:3\n")
            # 元素种类 位置 力
            for part1, part2 in zip(type_position, force):

                part2 = part2.split(' ')
                # 定义正则表达式模式，匹配整数和浮点数
                pattern = re.compile(r'^[+-]?\d+(\.\d+)?$')
                # 使用列表推导式过滤非数字元素
                part2 = [item for item in part2 if pattern.match(item)]
                part2 = '\t'.join(part2)

                part = '\t'.join([part1, part2])
                f.write(part+"\n")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：automation_nep
@File ：nep.py
@Author ：RongYi
@Date ：2025/4/30 19:35
@E-mail ：2071914258@qq.com
"""
import os


def train_nep_v0(config):
    with open("./nep-dataset/nep.in", 'w', encoding='utf-8') as f:
        element_type = config["element_type"]
        cutoff = config["nep"]["cutoff"]

        f.write(f"type {len(element_type)}")
        for ele in element_type:
            f.write(f" {ele}")

        f.write(f"\ncutoff")
        for cutoff in cutoff:
            f.write(f" {cutoff}")
    os.system(f'{config["nep"]["nep_path"]}')





#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：automation_nep
@File ：cli.py
@Author ：RongYi
@Date ：2025/4/29 14:31
@E-mail ：2071914258@qq.com
"""
import argparse
from train.train import train
from utils.config import load_config

def main():
    """
    命令行参数设置
    :return:
    """
    parser = argparse.ArgumentParser(description="Automation NEP CLI")
    subparsers = parser.add_subparsers(dest="command")

    # 添加 train 子命令
    train_parser = subparsers.add_parser("train", help="Run training")
    # 为 train 添加子命令 yaml 配置文件
    train_parser.add_argument("-yaml", default="train.yaml", help="train config file")

    args = parser.parse_args()
    if args.command == 'train':
        config_path = args.yaml
        config = load_config(config_path)
        train(config)
    else:
        parser.print_help()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：MyTools
@File ：io.py
@Author ：RongYi
@Date ：2025/4/28 22:56
@E-mail ：2071914258@qq.com
"""

import os
import re
from utils.myprint import myprint
from ase.io import read


def find_pp_basic(config):
    """
    寻找基组赝势文件
    :param config:
    :return:
    """
    pp, basis = {}, {}

    for index, element_type in enumerate(config['element_type']):
        pp_flag, basis_flag = False, False  # 标志位
        # pp_pattern: 以元素类型开头, 以.upf结尾
        pp_pattern = rf"^{element_type}.*upf$"
        # basis_pattern: 以元素类型开头, 以.orb结尾
        basis_pattern = rf"^{element_type}_.*orb$"

        for filename in os.listdir(config["abacus"]["pp_path"]):
            if re.match(pp_pattern, filename):
                pp[f'{element_type}'] = filename
                pp_flag = True
                break

        for filename in os.listdir(config["abacus"]["basis_path"]):
            if re.match(basis_pattern, filename):
                basis[f'{element_type}'] = filename
                basis_flag = True
                break

        if pp_flag is False:
            myprint(f"未找到 {element_type} 赝势文件, 请检查 {config["abacus"]["pp_path"]}", 'RED')
            exit()
        if basis_flag is False:
            myprint(f"未找到 {element_type} 轨道文件, 请检查 {config["abacus"]["basis_path"]}", 'RED')
            exit()
    myprint(f"赝势文件: {pp}", 'YELLOW')
    myprint(f"轨道文件: {basis}", 'YELLOW')
    return pp, basis


def abacus_dataset(config):
    """
    制作 abacus 训练集
    :param config: 训练配置
    :return: 数据集路径
    """
    # 存在 abacus_dataset 读取数据集大小
    if config["abacus"]["dataset_path"]:
        if os.path.exists(config["abacus"]["dataset_path"]):
            myprint(f'使用已存在数据集 {config["abacus"]["dataset_path"]}')
            dataset_roots = []
            # 寻找 STRU 统计数据集大小
            for root, _, files in os.walk(config["abacus"]["dataset_path"]):
                for file in files:
                    if file == 'STRU':
                        dataset_roots.append(os.path.join(root, file))
                        break

            myprint(f"数据集大小: {len(dataset_roots)}")
            return dataset_roots
        else:
            myprint(f"未找到 {config['abacus']['dataset_path']} 请检查配置文件", 'RED')
            exit()

    if not os.path.exists(config['init_train_xyz']):
        myprint(f"未找到 {config['init_train_xyz']} 请检查配置文件", 'RED')
        exit()
    # 自动生成 abacus dataset
    home_path = os.getcwd()
    dataset_roots = []
    atoms = read(config['init_train_xyz'])
    # 寻找赝势文件和轨道文件
    pp, basis = find_pp_basic(config)
    myprint(f"生成训练集 abacus_dataset 训练集大小: {len(atoms)}")
    for i in range(1, len(atoms) + 1):
        os.makedirs(f'./abacus_dataset/{i}', exist_ok=True)
        atoms.write(f'./abacus_dataset/{i}/STRU', format='abacus', pp=pp,
                    basis=basis)
        dataset_roots.append(os.path.join(home_path, f"./abacus_dataset/{i}/STRU"))
    return dataset_roots




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：automation_nep
@File ：test.py
@Author ：RongYi
@Date ：2025/5/1 1:13
@E-mail ：2071914258@qq.com
"""
import os
root = r"D:/Papers/MyTools/automation_nep/abacus-dataset/0-99/1/STRU"
print('/'.join(root.split("/")[:-1])+"/time.json")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：automation_nep
@File ：config.py
@Author ：RongYi
@Date ：2025/4/29 16:07
@E-mail ：2071914258@qq.com
"""
import yaml


def load_config(config_path):
    """
    加载 YAML 配置文件
    :param config_path: YAML 文件路径
    :return: 配置字典
    """
    with open(config_path, 'r', encoding='utf-8') as file:
        config = yaml.safe_load(file)
    return config




#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：MyTools
@File ：get_now_time.py
@Author ：RongYi
@Date ：2025/4/29 10:42
@E-mail ：2071914258@qq.com
"""
from datetime import datetime
from colorama import Fore, Style


def now_time():
    """
    获取当前时间
    :return: 当前时间
    """
    now = datetime.now()
    formatted_time = now.strftime("%Y-%m-%d %H:%M:%S")
    return Fore.CYAN+formatted_time+Style.RESET_ALL


def myprint(content: str, color='WHITE'):
    if color == 'RED':
        return print(f"[{now_time()}] {Fore.RED + content + Style.RESET_ALL}")
    elif color == 'YELLOW':
        return print(f"[{now_time()}] {Fore.YELLOW + content + Style.RESET_ALL}")
    else:
        return print(f"[{now_time()}] {content}")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：auto_nep
@File ：setup.py.py
@Author ：RongYi
@Date ：2025/5/1 12:53
@E-mail ：2071914258@qq.com
"""
from setuptools import setup, find_packages

setup(
    name="auto-nep",
    version="0.0.3",
    author='Jack-RY',
    author_email='2071914258@qq.com',
    packages=find_packages(),
    entry_points={
        'console_scripts': [
            'auto-nep= scr.auto_nep.cli.cli:main',
        ],
    },
)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
"""
@Project ：automation_nep
@File ：sub_task.py
@Author ：RongYi
@Date ：2025/4/29 20:48
@E-mail ：2071914258@qq.com
"""
import os
import re
import time
import shutil
from colorama import Style
from utils.myprint import myprint


def sub_task(config, dataset_roots):
    """
    提交任务脚本
    :param config: 配置文件
    :return:
    """
    # 检查文件
    if not os.path.exists(config[f"abacus"]["pbs_path"]):
        myprint(f'未找到 {config["abacus"]["pbs_path"]} 请检查配置文件', 'RED')
        exit()
    if not os.path.exists(config["abacus"]["input_path"]):
        myprint(f'未找到 {config["abacus"]["input_path"]} 请检查配置文件', 'RED')
        exit()
    # 检查数据集路径
    if config["abacus"]["dataset_path"]:
        dataset_path = config["abacus"]["dataset_path"]
    else:
        dataset_path = "./abacus_dataset"

    # 提交任务: 收敛标准 time.json文件
    task_num = 0
    home_path = os.getcwd()
    for root in dataset_roots:
        if not os.path.exists('/'.join(root.split("/")[:-1]) + "/time.json"):
            os.chdir('/'.join(root.split("/")[:-1]))
            os.system('qsub abacus.pbs')
            shutil.copy(home_path + config["abacus"]["input_path"], '.')
            shutil.copy(home_path + config["abacus"]["pbs_path"], '.')
            task_num += 1
    # 提交完成后退回主目录
    os.chdir(home_path)
    myprint(f"任务提交完成 提交计算任务: {task_num}")


def spend_time(task_path):
    """
    读取当前任务所花费时间
    :param task_path: 任务路径
    :return: step time: h m s
    """
    with open(task_path + "/out.log", encoding='utf-8') as f:
        content = f.read()
        time_pattern = re.compile(r" CU\d+\s+.*\s+(\d+\.\d+)$", re.MULTILINE)
        time_match = time_pattern.findall(content)
        spend_time = 0
        for time in time_match:
            spend_time += float(time)
        step_pattern = re.compile(r" CU\d+\s+.*\s+\d+\.\d+$", re.MULTILINE)
        step_match = step_pattern.findall(content)
        return step_match[-1], str(int(spend_time // 3600)), str(int(spend_time // 60 % 60)), str(round(spend_time % 60))


def check_task(config, dataset_roots):
    """
    检测任务是否完成
    1.有 out.log 无 time.json 计算中
    2.有 time.json 计算完成
    3. 无 out.log 无 time.json 等待中
    :return:
    """
    start_time = time.perf_counter()
    total_time = 0  # 计算总耗时 min

    # 检查数据集路径
    if config["abacus"]["dataset_path"]:
        dataset_path = config["abacus"]["dataset_path"]
    else:
        dataset_path = "./abacus_dataset"
    # 处理未计算任务
    warn_times = 1
    while True:
        accomplish = []
        calculating = []
        awating = []
        for root in dataset_roots:
            time_json = '/'.join(root.split("/")[:-1]) + "/time.json"
            out_log = '/'.join(root.split("/")[:-1]) + f"/out.log"
            if os.path.isfile(time_json):
                accomplish.append('/'.join(root.split("/")[:-1]))
            elif os.path.isfile(out_log):
                calculating.append('/'.join(root.split("/")[:-1]))
            else:
                awating.append('/'.join(root.split("/")[:-1]))

        if total_time % 5 == 0:
            # Current Task 打印模块
            myprint("\n-------------------------------- abacus -------------------------------\n"
                   f"Total task num: {len(dataset_roots)}\t Total time(s): {round(time.perf_counter() - start_time, 2)}\t  Progress:{len(accomplish)}/{dataset_length}\n"
                   f"-----------------------------------------------------------------------")
            for task in calculating:
                step, h, m, s = spend_time(task)
                print(f"Current Task: [{task}] Spend Time: [{h}h {m}m {s}s]\n"
                      f"Step: [{step}]\n"
                      f"-----------------------------------------------------------------------")

        if len(accomplish) == len(dataset_roots):
            myprint("计算完成提取 nep 训练集 train.xyz", 'RED')
            myprint(f"Mean time(s):{(time.perf_counter() - start_time)/len(accomplish): .2f} s")
            break

        if len(calculating) == 0 and len(awating) > 0:
            myprint(f"Warning {warn_times}: 以下任务未进行计算!", "RED")
            for task in awating:
                print(f"{task + Style.RESET_ALL}")
            warn_times += 1
            if warn_times > config["abacus"]["warn_times"]:
                # 警告超过三次 自动退出 进行下一步
                break

        total_time += 1
        time.sleep(60)

